/**
 * Back 2 Browser Bytecode Translator
 * Copyright (C) 2012 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://opensource.org/licenses/GPL-2.0.
 */

package org.apidesign.bck2brwsr.htmlpage;

import java.lang.reflect.Method;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public final class KOProperty {
    private static final Logger LOG = Logger.getLogger(KOProperty.class.getName());
    private final Object obj;
    private final Method getter;
    private final Method setter;
    
    KOProperty(Object obj, String getter, String setter) throws NoSuchMethodException {
        this.obj = obj;
        this.getter = obj.getClass().getDeclaredMethod(getter);
        this.getter.setAccessible(true);
        if (setter != null) {
            this.setter = obj.getClass().getDeclaredMethod(setter, this.getter.getReturnType());
            this.setter.setAccessible(true);
        } else {
            this.setter = null;
        }
    }
    
    public Object get() throws Exception {
        Object v = getter.invoke(obj);
        LOG.log(Level.INFO, "{0} returns {1}", new Object[] { getter, v });
        return v;
    }
    
    public void set(Object value) throws Exception {
        setter.invoke(obj, value);
    }
}
